
package io_handle;

import java.io.*;
import java.net.*;


public class ListenSocket implements Handle {
    protected ServerSocket socket;
    protected boolean ready_, closed_;
    protected Socket conn_socket;
    protected IOException exception;


    public ListenSocket(int port) throws IOException {
	socket = new ServerSocket(port);
	new ListenThread(this).start();
    }


    public synchronized StreamSocket accept() throws IOException {
	while (!ready_) {
	    try { wait(); } catch (InterruptedException e) { }
	}
	Socket s = conn_socket;
	IOException e = exception;
	conn_socket = null;
	exception = null;
	ready_ = false;
	notifyAll();
	if (s == null) {
	    throw e;
	}
	return new StreamSocket(s);
    }


    public boolean ready() {
	return ready_;
    }


    public boolean closed() {
	return closed_;
    }


    public void close() throws IOException {
	socket.close();
	closed_ = true;
	ready_ = false;
	socket = null;
	conn_socket = null;
    }


    synchronized void put (Socket sock, IOException exc) {
	while (ready_) {
	    try { wait(); } catch (InterruptedException e) { }
	}
	conn_socket = sock;
	exception = exc;
	ready_ = true;
	SynchronousDemultiplexer.instance().notify_event(this);
	notify();
    }

}


class ListenThread extends Thread {
    private ListenSocket handle;


    ListenThread(ListenSocket h) {
        handle = h;
    }


    public void run() {
	while (! handle.closed_) {
	    try {
		handle.put(handle.socket.accept(),null);
	    } catch (IOException e) {
		handle.put(null,e);
	    }
	}
    }

}
